<?php
// Copyright 2003 Won-Kyu Park <wkpark at kldp.org>. All rights reserved.
// Distributable under GPL. See COPYING for more information.
// $Id: monisetup.php,v 1.13 2004/01/15 04:16:59 verotas Exp $

class MoniConfig {
  function MoniConfig($configfile="config.php") {
    $this->configfile = $configfile;
    $this->_getRawConfig($configfile);
    if (!$this->rawconfig) {
      $this->_getDefaultRawConfig();
    }
    $this->_rawconfig2config();
  }

  function _getDefaultRawConfig() {
    $this->_getRawConfig();
    $hostconfig=$this->_getHostConfig();
    $this->rawconfig = array_merge($this->rawconfig, $hostconfig);
  }

  function _rawconfig2config()
  {
    while (list($key, $val) = each($this->rawconfig)) {
      eval("\$$key=$val;");
      eval("\$this->config[\$key]=$val;");
    }
  }

  function _getHostConfig() {
    if (function_exists("dba_open")) {
      $tempnam="/tmp/".time();
      if ($db=@dba_open($tempnam,"n","db3"))
        $config['dba_type']="'db3'";
      else if ($db=@dba_open($tempnam,"n","db2"))
        $config['dba_type']="'db2'";
      else if ($db=@dba_open($tempnam,"n","gdbm"))
        $config['dba_type']="'gdbm'";

      if ($db) {
        dba_close($db);
      }
    }

    preg_match("/Apache\/2\.0\./",$_SERVER['SERVER_SOFTWARE'],$match);
    if ($match) {
      $config['query_prefix']='"?"';
      $config['kbd_script']='$url_prefix."/css/kbd2.js"';
    }

    $url_prefix=preg_replace("/\/([^\/]+)\.php$/","",$_SERVER['SCRIPT_NAME']);
    $config['url_prefix']="'".$url_prefix."'";

    $config['rcs_user']="'".getenv('LOGNAME')."'";
    return $config;
  }

  function setConfig($config) {
    $this->config=$config;
  }

  function setRawConfig($rawconfig) {
    $this->rawconfig=$rawconfig;
  }

  function setFormConfig($formconfig)
  {
    $config=array();
    while (list($key, $val) = each($formconfig)) {
      $val=stripslashes($val);
      if ($val == "") $val="''";
      eval("\$dum=$val;");
      eval("\$$key=$val;");
      //$config[$key]=$dum;
      $rawconfig[$key]=$val;
    }
    //$this->setConfig($config);
    $this->setRawConfig($rawconfig);
  }

  function _getConfig($configfile) {
    if (!is_readable($configfile)) {
      return array();
    }
    $org=array();
    $org=get_defined_vars();
    include($configfile);
    $new=get_defined_vars();

    return array_diff($new, $org);
  }

  function _getRawConfig($configfile='config.php.default') {
    if (!is_readable($configfile)) {
      return array();
    }
    $lines=file($configfile);
    foreach ($lines as $line) {
      if ($line[0] != '$') {
        continue;
      }
      $d = explode("=", substr($line, 1), 2);
      if ($d[0]) {
        $val = preg_replace("/\s*;$/", "", trim($d[1]));
        $config[$d[0]] = $val;
      }
    }
    $this->rawconfig = $config;
  }

  function genRawConfig($rawconfig)
  {
    $lines = array("<?php\n", "# automatically generated by monisetup.php\n");
    while (list($key, $val) = each($rawconfig)) {
      if (($key == 'admin_passwd') or ($key == 'purge_passwd')) {
        $val = "'".crypt($val,md5(time()))."'";
      }
      $lines[] = "\$$key=$val;\n";
    }
    $lines[] = "?>\n";
    $rawconf = join("", $lines);
    return $rawconf;
  }

  function putRawConfig($rawconfig)
  {
    $ret = FALSE;
    if ($this->config_writable()) {
      $rawconf = $this->genRawConfig($rawconfig);
      umask(000);
      $fp = fopen($this->configfile, "w");
      fwrite($fp, $rawconf);
      fclose($fp);
      @chmod($this->configfile, 0666);
      $ret = TRUE;
    }
    return $ret;
  }

  function config_writable()
  {
    $ret = FALSE;
    if (is_writable($this->configfile)) {
      $ret = TRUE;
    } elseif (!file_exists($this->configfile) and is_writable(".")) {
      $ret = TRUE;
    }
    return $ret;
  }

  function showConfig()
  { // display $this->config
  }

  function showRawConfig()
  { // display $this->rawconfig
  }
}

function check_dir($config) { //FIXME: refactor w/ monisetup.sh
  if (!is_writable($config['data_dir'])) {
    if (02000 & fileperms(".")) { # check sgid
      $datadir_perm = 0775;
    } else {
      $datadir_perm = 0777;
    }
    $datadir_perm = decoct($datadir_perm);
    print "<h3><font color=red>FATAL: $config[data_dir] directory is not writable</font></h3>\n";
    print "<h4>Please execute the following command</h4>";
    print "<pre class='console'>\n".
          "<font color='green'>$</font> chmod $datadir_perm $config[data_dir]\n".
          "</pre>\n";
    return FALSE;
  }

  $data_sub_dir=array("cache","user","text","text/RCS");
  if (02000 & fileperms($config['data_dir'])) {
    $DPERM=0775;
  } else {
    $DPERM=0777;
  }

  foreach($data_sub_dir as $dir) {
    if (!file_exists("$config[data_dir]/$dir")) {
      umask(000);
      mkdir("$config[data_dir]/$dir",$DPERM);
      if ($dir == 'text') {
        mkdir($config['data_dir']."/$dir/RCS",$DPERM);
      }
    } elseif (!is_writable("$config[data_dir]/$dir")) {
      print "<h4><font color=red>$dir directory is not writable</font></h4>\n".
            "<pre class='console'>\n".
            "<font color='green'>$</font> chmod a+w $config[$file]\n".
            "</pre>\n";
    }
  }

  $writables=array("upload_dir", "editlog_name");
  foreach($writables as $file) {
    if (!is_writable($config[$file])) {
      if (file_exists($config[$file])) {
        print "<h3><font color=red>$config[$file] is not writable</font> :( </h3>\n";
        print "<pre class='console'>\n".
              "<font color='green'>$</font> chmod a+w $config[$file]\n".
              "</pre>\n";
      } else {
        if (preg_match("/_dir/",$file)) {
          umask(000);
          mkdir($config[$file],$DPERM);
          print "<h3><font color=blue>$config[$file] is created now</font> :)</h3>\n";
        } else {
          $fp=@fopen($config[$file],"w+");
          if ($fp) {
            fclose($fp);
            chmod($config[$file],0666);
            print "<h4><font color='green'>$config[$file] is created now</font> ;) </h4>\n";
          } else {
            print "<pre class='console'>\n".
            "<font color='green'>$</font> touch $config[$file]\n".
            "<font color='green'>$</font> chmod a+w $config[$file]\n</pre>\n";
          }
        }
      }
    } else {
//      print "<h3><font color=blue>$config[$file] is writable</font> :)</h3>\n";
    }
  }
  return TRUE;
}

function keyToPagename($key) {
#  return preg_replace("/_([a-f0-9]{2})/e","chr(hexdec('\\1'))",$key);
  $pagename=preg_replace("/_([a-f0-9]{2})/","%\\1",$key);
#  $pagename=str_replace("_","%",$key);
#  $pagename=strtr($key,"_","%");
  return rawurldecode($pagename);
}

function pagenameToKey($pagename) {
  return preg_replace("/([^a-z0-9]{1})/ie","'_'.strtolower(dechex(ord('\\1')))",$pagename);
}

function show_config_table($config, $rawconfig)
{
  print_title("Current settings for $config[sitename]");
  print "<table align=center border=1 cellpadding=2 cellspacing=2>\n";
  print "<tr>";
  print "<td align='center'>variable</td>";
  print "<td align='center'>value</td>";
  print "<td align='center'>result</td>";
  print "</tr>\n";
  while (list($key, $val) = each($config)) {
    if ($key != "admin_passwd" && $key != "purge_passwd") {
      $val=htmlspecialchars($val);
      print "<tr>";
      print "<td>$$key</td>";
      print "<td>".htmlspecialchars($rawconfig[$key])."</td>";
      if ($config[$key]==='')
         $ret="<br />";
      else
         $ret=$config[$key];
      print "<td>$ret</td>";
      print "</tr>\n";
    }
  }
  print "</table>\n";
  print "<br/>\n";
}

function show_config_edit_form($config, $rawconfig)
{
  print_title("Change your configurations");
  print "<form method='post'>\n";
  print "<table align=center border=1 cellpadding=2 cellspacing=2>\n";
  print "<tr>";
  print "<td align='center'>variable</td>";
  print "<td align='center'>value</td>";
  print "<td align='center'>result</td>";
  print "</tr>\n";
  while (list($key, $val) = each($rawconfig)) {
    if ($key != "admin_passwd") {
      $val=htmlspecialchars($val);
      print "<tr>";
      print "<td>$$key</td>";
      print "<td><input name='config[$key]' value=\"$val\" size='40'></td>";
      if ($config[$key]==='')
         $ret="<br />";
      else
         $ret=$config[$key];
      print "<td>$ret</td>";
      print "</tr>\n";
    }
  }

  if (!$config['admin_passwd']) {
    print "<tr><td><b>\$admin_passwd</b></td>";
    print "<td colspan='2'><input type='password' name='newpasswd' size='40'></td></tr>\n";
  } else  {
    print "<tr><td><b>Old password</b></td>";
    print "<td><input type='password' name='oldpasswd' size='40'></td></tr>\n";
    print "<tr><td><b>New password</b></td>";
    print "<td><input type='password' name='newpasswd' size='40'></td></tr>\n";
  }
  print "<tr><td colspan=3>";
  print "<input type='submit' name='action' value='preview'/>";
  print "<input type='submit' name='action' value='update' />";
  print "</td></tr>\n";
  print "</table>\n";
  print "</form>\n";
}


//
// do_action()'s
//

function do_show($Config)
{
  $config=$Config->config;
  $rawconfig=$Config->rawconfig;

  if (!file_exists("config.php")) { /* maybe this is the first run */
    print_title("Welcome! This is your first installation.");
    print_notice("Default settings are loaded...");
    $ret = $Config->putRawConfig($rawconfig);
    if ($ret) {
      print_notice("Initial configurations are saved successfully.");
    } else {
      print_error("Saving new configurations failed!!");
      return;
    }
  }

  if (!file_exists($config[data_dir]."/text/RecentChanges")) {
    print_notice("You have no WikiSeed on your $config[sitename]");
    print_notice("You could <a href='?action=seed'>sow WikiSeed's now.</a>");
  }

  show_config_table($config, $rawconfig);
}

function do_edit($Config)
{
  $config=$Config->config;
  $rawconfig=$Config->rawconfig;

  if (!$config['admin_passwd']) {
    print_warn("You have to enter your admin_password");
  }
  show_config_edit_form($config, $rawconfig);

  return;
}

function do_update($Config)
{
  $Config->setFormConfig($_POST['config']);
  $config=$Config->config;
  $rawconfig=$Config->rawconfig;

  $passwd = $Config->config['admin_passwd'];
  $oldpasswd = $_POST['oldpasswd'];
  $newpasswd = $_POST['newpasswd'];
  $is_valid_passwd = TRUE;
  if ($passwd) {
    if (crypt($oldpasswd, $passwd) == $passwd) {
      $is_valid_passwd = TRUE;
    } else {
      print_error("Invalid password !!");
      print_error("If you can't remember the password, delete \$admin_password in 'config.php' and restart.");
      $is_valid_passwd = FALSE;
    }
  }

  if ($is_valid_passwd) {
    if ($newpasswd) {
      //print_debug("setting newpasswd = '$newpasswd'");
      $rawconfig['admin_passwd'] = $newpasswd;
    } else {
      //print_debug("setting oldpasswd = '$oldpasswd'");
      $rawconfig['admin_passwd'] = $oldpasswd;
    }
  }

  $rawconf = $Config->genRawConfig($rawconfig);
  if ($is_valid_passwd) {
    $ret = $Config->putRawConfig($rawconfig);
    if ($ret) {
      print_notice("Configurations are saved successfully.");
      print_warn("Please execute the following command in the console.");
      print_pre("\$ sh secure.sh");
      print_title("Updated Configurations for this $config[sitename]");
      print "<pre class='console'>\n";
      highlight_string($rawconf);
      print "</pre>\n";
    } else {
      print_error("Saving configurations failed !!");
    }
  }

  return;
}

function do_preview($Config)
{
  $Config->setFormConfig($_POST['config']);
  $config=$Config->config;
  $rawconfig=$Config->rawconfig;
  show_config_table($config, $rawconfig);
  return;
}

function do_cancel($Config)
{
  print_title("You have canceled something.");
  return;
}

function do_seed($Config)
{
  $seeddir='wikiseed';
  $pages= array();
  $handle= opendir($seeddir);
  while ($file = readdir($handle)) {
    if (is_dir($seeddir."/".$file)) continue;
    $pagename = keyToPagename($file);
    $pages[$pagename] = $pagename;
  }
  closedir($handle);
#  sort($pages);
  $idx=1;

  $num=sizeof($pages);

  $SystemPages="FrontPage|RecentChanges|TitleIndex|WordIndex|FindPage|".
  "FortuneCookies|Pages$|".
  "SystemPages|TwinPages|WikiName|SystemInfo|UserPreferences|".
  "InterMap|IsbnMap|WikiSandBox|SandBox|UploadFile|UploadedFiles|".
  "InterWiki|SandBox|".
  "BlogChanges|HotDraw|OeKaki";

  $WikiTag='DeleteThisPage';
  $seed_filters= array(
    "HelpPages"=>array('/^Help.*/',1),
    "Category pages"=>array('/^Category.*/',1),
    "Macro pages"=>array('/Macro$/',1),
    "MoniWiki pages"=>array('/MoniWiki.*|Moni/',1),
    "MoinMoin pages"=>array('/MoinMoin.*/',1),
    "Templates"=>array('/Template$/',1),
    "SystemPages"=>array("/($SystemPages)/",1),
    "WikiTags"=>array("/($WikiTag)/",1),
    "Wiki etc."=>array('/Wiki/',1),
    "Misc."=>array('//',0),
  );

  $wrap=1;

  print "<h3>Total $num pages found</h3>\n";
  print "<form method='post' action=''>\n";
  while (list($filter_name,$filter) = each($seed_filters)) {
    print "<h4>$filter_name</h4>\n";
    foreach ($pages as $pagename) {
      if (preg_match($filter[0],$pagename)) {
        print "<input type='checkbox' name='seeds[$idx]' value='$pagename'";
        if ($filter[1])
          print "checked='checked' />$pagename ";
        else
          print " />$pagename ";
        $idx++;
        if ($wrap++ % 4 == 0) print "<br />\n";
        unset($pages[$pagename]);
      }
    }
    $wrap=1;
  }
  print "<input type='hidden' name='action' value='sow_seed' />\n";
  print "<br /><input type='submit' value='sow WikiSeeds'></form>\n";

  return;
}

function do_sow_seed($Config)
{
  $seeds = $_POST['seeds'];
  if (!$seeds) {
    print_error("No WikiSeeds are selected.");
    return FALSE;
  }
  $seeddir = 'wikiseed';
  $textdir = $Config->config['text_dir'];
  umask(000);
  print "<pre class='console'>\n";
  foreach($seeds as $seed) {
    $pagekey=pagenameToKey($seed);
    $cmd="cp $seeddir/$pagekey $textdir]";
    print $cmd."\n";
    #system(escapeshellcmd($cmd));
    copy("$seeddir/$pagekey", $textdir."/$pagekey");
  }
  print "</pre>\n";

  print_notice("WikiSeeds are sowed successfully");
  return TRUE;
}

//
// print_*()
//

function print_header()
{
  print <<<HEADER
<html>
<head>
<title>MoniWiki Setup</title>
<link rel="stylesheet" type="text/css" href="css/default.css"/>
</head>
<body>
<h2>
<a href='monisetup.php'>
<img src="imgs/moniwiki-logo.gif" alt="MoniWiki" border="0" align="middle" />
&nbsp;Setup
</a>
&nbsp;:&nbsp;
<a href='?action=show'>show</a>&nbsp;
<a href='?action=edit'>edit</a>&nbsp;
<a href='?action=seed'>seed</a>&nbsp;
</h2>
HEADER;
}

function print_footer()
{
  print <<<FOOTER
</body>
</html>
FOOTER;
}

function print_msg($class, $msg)
{
}

function print_error($msg)
{
  print "<h3><font color='red'>";
  print "ERROR : ";
  print $msg;
  print "</font></h3>\n";
}

function print_warn($msg)
{
  print "<h3><font color='magenta'>";
  print "WARNING : ";
  print $msg;
  print "</font></h3>\n";
}

function print_title($msg)
{
  print "<h3><font color='blue'>";
  print $msg;
  print "</font></h3>\n";
}

function print_notice($msg)
{
  print "<h3><font color='green'>";
  print "NOTICE : ";
  print $msg;
  print "</font></h3>\n";
}

function print_pre($msg)
{
  print "<pre class='console'>\n";
  print $msg . "\n";
  print "</pre>\n";
}

function print_debug($msg)
{
  print "<h4><font color='grey'>";
  print "DEBUG : ";
  print $msg;
  print "</font></h4>\n";
}

//
// main function
//

function setup_main()
{
  $action = $_GET['action'] or $_POST['action'];
  if (!$action) {
    $action = "show";
  }

  print_header();

  $Config = new MoniConfig();
  if (!$Config->config_writable()) {
    print_error("Please execute the following command to change your configuration.");
    print_pre("\$ sh monisetup.sh");
  }
  if (!check_dir($Config->config)) {
    print_error("There seems to be some problem. Please excute the following command.");
    print_pre("\$ sh monisetup.sh");
  }

  //ob_start();
  //print_body()
  if (function_exists("do_$action")) {
    call_user_func("do_$action", $Config);
  } else {
    print_error("Unknown action '$action'.");
  }
  //ob_end_flush();

  print_footer();

  return;
}

setup_main();

// vim:ts=8:sts=2:sw=2:et
?>
